@@ -7,69 +7,105 @@ use std::rc::Rc;
 use std::sync::Arc;
 
 /// **B**orrowed, **O**wned or **S**hared smart pointer.
-#[non_exhaustive]
 pub enum Bos<'b, B>
 where
     B: 'b + ToOwned + ?Sized,
 {
     Borrowed(&'b B),
-    Owned(<B as ToOwned>::Owned),
-    Arc(Arc<B>),
-    Rc(Rc<B>),
+    Owned(B::Owned),
+    ArcBorrowed(Arc<B>),
+    ArcOwned(Arc<B::Owned>),
+    RcBorrowed(Rc<B>),
+    RcOwned(Rc<B::Owned>),
 }
 
 impl<'b, B: 'b + ToOwned + ?Sized> Bos<'b, B> {
-    pub fn into_owned(self) -> <B as ToOwned>::Owned {
+    /// Extracts the owned data.
+    ///
+    /// Clones the data if it is not already owned.
+    #[inline]
+    pub fn into_owned(self) -> B::Owned {
         match self {
             Bos::Borrowed(x) => x.to_owned(),
             Bos::Owned(x) => x,
-            Bos::Arc(x) => x.as_ref().to_owned(),
-            Bos::Rc(x) => x.as_ref().to_owned(),
+            Bos::ArcBorrowed(x) => x.as_ref().to_owned(),
+            Bos::ArcOwned(x) => {
+                Arc::try_unwrap(x).unwrap_or_else(|x| Borrow::borrow(&*x).to_owned())
+            }
+            Bos::RcBorrowed(x) => x.as_ref().to_owned(),
+            Bos::RcOwned(x) => Rc::try_unwrap(x).unwrap_or_else(|x| Borrow::borrow(&*x).to_owned()),
         }
     }
 
+    /// This is unstable and only available with the `unstable` feature.
+    #[cfg(any(doc, feature = "unstable"))] // reason: recently added, maybe we should add a check, if Borrowed should become Shared instead?
+    #[inline]
     pub fn into_static(self) -> Bos<'static, B> {
         match self {
             Bos::Borrowed(x) => Bos::Owned(x.to_owned()),
             Bos::Owned(x) => Bos::Owned(x),
-            Bos::Arc(x) => Bos::Arc(x),
-            Bos::Rc(x) => Bos::Rc(x),
+            Bos::ArcBorrowed(x) => Bos::ArcBorrowed(x),
+            Bos::ArcOwned(x) => Bos::ArcOwned(x),
+            Bos::RcBorrowed(x) => Bos::RcBorrowed(x),
+            Bos::RcOwned(x) => Bos::RcOwned(x),
         }
     }
 
-    pub fn to_mut(&mut self) -> &mut <B as ToOwned>::Owned {
+    /// Acquires a mutable reference to the owned form of the data.
+    ///
+    /// Clones the data if it is not already owned.
+    #[inline]
+    pub fn to_mut(&mut self) -> &mut B::Owned
+    where
+        B::Owned: Clone,
+    {
         match self {
             Bos::Borrowed(x) => {
                 *self = Bos::Owned(x.to_owned());
                 self.to_mut()
             }
-            Bos::Owned(o) => o,
-            Bos::Arc(x) => {
+            Bos::Owned(x) => x,
+            Bos::ArcBorrowed(x) => {
                 *self = Bos::Owned(x.as_ref().to_owned());
                 self.to_mut()
             }
-            Bos::Rc(x) => {
+            Bos::ArcOwned(x) => Arc::make_mut(x),
+            Bos::RcBorrowed(x) => {
                 *self = Bos::Owned(x.as_ref().to_owned());
                 self.to_mut()
             }
+            Bos::RcOwned(x) => Rc::make_mut(x),
+        }
+    }
+
+    #[inline]
+    fn internal_borrow(&self) -> &B {
+        match self {
+            Bos::Borrowed(x) => x,
+            Bos::Owned(x) => x.borrow(),
+            Bos::ArcBorrowed(x) => x.deref(),
+            Bos::ArcOwned(x) => x.deref().borrow(),
+            Bos::RcBorrowed(x) => x.deref(),
+            Bos::RcOwned(x) => x.deref().borrow(),
         }
     }
 }
 
 impl<'b, B: 'b + ToOwned + ?Sized, Rhs> Add<Rhs> for Bos<'b, B>
 where
-    <B as ToOwned>::Owned: Add<Rhs>,
+    B::Owned: Add<Rhs>,
 {
-    type Output = <<B as ToOwned>::Owned as Add<Rhs>>::Output;
+    type Output = <B::Owned as Add<Rhs>>::Output;
 
-    fn add(self, rhs: Rhs) -> <<B as ToOwned>::Owned as Add<Rhs>>::Output {
+    #[inline]
+    fn add(self, rhs: Rhs) -> <B::Owned as Add<Rhs>>::Output {
         self.into_owned() + rhs
     }
 }
 
 impl<'b, B: 'b + ToOwned + ?Sized, Rhs> AddAssign<Rhs> for Bos<'b, B>
 where
-    <B as ToOwned>::Owned: AddAssign<Rhs>,
+    B::Owned: AddAssign<Rhs> + Clone,
 {
     #[inline]
     fn add_assign(&mut self, rhs: Rhs) {
@@ -80,33 +116,30 @@ where
 impl<'b, B: 'b + ToOwned + ?Sized> AsRef<B> for Bos<'b, B> {
     #[inline]
     fn as_ref(&self) -> &B {
-        self.borrow()
+        self.internal_borrow()
     }
 }
 
 impl<'b, B: 'b + ToOwned + ?Sized> Borrow<B> for Bos<'b, B> {
     #[inline]
     fn borrow(&self) -> &B {
-        match self {
-            Self::Borrowed(x) => x,
-            Self::Owned(x) => x.borrow(),
-            Self::Arc(x) => x.as_ref().borrow(),
-            Self::Rc(x) => x.as_ref().borrow(),
-        }
+        self.internal_borrow()
     }
 }
 
 impl<'b, B: 'b + ToOwned + ?Sized> Clone for Bos<'b, B>
 where
-    <B as ToOwned>::Owned: Clone,
+    B::Owned: Clone,
 {
     #[inline]
     fn clone(&self) -> Self {
         match self {
             Self::Borrowed(x) => Self::Borrowed(x),
             Self::Owned(x) => Self::Owned(x.clone()),
-            Self::Arc(x) => Self::Arc(x.clone()),
-            Self::Rc(x) => Self::Rc(x.clone()),
+            Self::ArcBorrowed(x) => Self::ArcBorrowed(x.clone()),
+            Self::ArcOwned(x) => Self::ArcOwned(x.clone()),
+            Self::RcBorrowed(x) => Self::RcBorrowed(x.clone()),
+            Self::RcOwned(x) => Self::RcOwned(x.clone()),
         }
     }
 }
@@ -122,26 +155,28 @@ impl<'b, B: 'b + ToOwned + ?Sized> Deref for Bos<'b, B> {
 
 impl<'b, B: 'b + ToOwned + ?Sized> Default for Bos<'b, B>
 where
-    <B as ToOwned>::Owned: Default,
+    B::Owned: Default,
 {
     /// Creates an owned Bos with the default value for the contained owned type.
     #[inline]
     fn default() -> Self {
-        Self::Owned(<B as ToOwned>::Owned::default())
+        Self::Owned(B::Owned::default())
     }
 }
 
 impl<'b, B: 'b + ToOwned + ?Sized> fmt::Debug for Bos<'b, B>
 where
     B: fmt::Debug,
-    <B as ToOwned>::Owned: fmt::Debug,
+    B::Owned: fmt::Debug,
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Bos::Borrowed(x) => f.debug_tuple("Bos::Borrowed").field(x).finish(),
             Bos::Owned(x) => f.debug_tuple("Bos::Owned").field(x).finish(),
-            Bos::Arc(x) => f.debug_tuple("Bos::Arc").field(x).finish(),
-            Bos::Rc(x) => f.debug_tuple("Bos::Rc").field(x).finish(),
+            Bos::ArcBorrowed(x) => f.debug_tuple("Bos::ArcBorrowed").field(x).finish(),
+            Bos::ArcOwned(x) => f.debug_tuple("Bos::ArcOwned").field(x).finish(),
+            Bos::RcBorrowed(x) => f.debug_tuple("Bos::RcBorrowed").field(x).finish(),
+            Bos::RcOwned(x) => f.debug_tuple("Bos::RcOwned").field(x).finish(),
         }
     }
 }
@@ -149,14 +184,16 @@ where
 impl<'b, B: 'b + ToOwned + ?Sized> fmt::Display for Bos<'b, B>
 where
     B: fmt::Display,
-    <B as ToOwned>::Owned: fmt::Display,
+    B::Owned: fmt::Display,
 {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Self::Borrowed(x) => fmt::Display::fmt(x, f),
             Self::Owned(x) => fmt::Display::fmt(x, f),
-            Self::Arc(x) => fmt::Display::fmt(x, f),
-            Self::Rc(x) => fmt::Display::fmt(x, f),
+            Self::ArcBorrowed(x) => fmt::Display::fmt(x, f),
+            Self::ArcOwned(x) => fmt::Display::fmt(x, f),
+            Self::RcBorrowed(x) => fmt::Display::fmt(x, f),
+            Self::RcOwned(x) => fmt::Display::fmt(x, f),
         }
     }
 }
@@ -165,9 +202,9 @@ impl<'b, B: 'b + ToOwned + ?Sized> Hash for Bos<'b, B>
 where
     B: Hash,
 {
+    #[inline]
     fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
-        let b: &B = self.borrow();
-        b.hash(state)
+        self.internal_borrow().hash(state)
     }
 }
 
@@ -175,9 +212,9 @@ impl<'b, B: 'b + ToOwned + ?Sized> PartialEq<B> for Bos<'b, B>
 where
     B: PartialEq,
 {
+    #[inline]
     fn eq(&self, other: &B) -> bool {
-        let b: &B = self.borrow();
-        b == other
+        self.internal_borrow() == other
     }
 }
 
@@ -185,10 +222,9 @@ impl<'b, B: 'b + ToOwned + ?Sized> PartialEq for Bos<'b, B>
 where
     B: PartialEq,
 {
+    #[inline]
     fn eq(&self, other: &Self) -> bool {
-        let b: &B = self.borrow();
-        let other_b: &B = other.borrow();
-        b == other_b
+        self.internal_borrow() == other.internal_borrow()
     }
 }
 
@@ -198,9 +234,9 @@ impl<'b, B: 'b + ToOwned + ?Sized> PartialOrd<B> for Bos<'b, B>
 where
     B: PartialOrd,
 {
+    #[inline]
     fn partial_cmp(&self, other: &B) -> Option<Ordering> {
-        let b: &B = self.borrow();
-        b.partial_cmp(other)
+        self.internal_borrow().partial_cmp(other)
     }
 }
 
@@ -208,10 +244,9 @@ impl<'b, B: 'b + ToOwned + ?Sized> PartialOrd for Bos<'b, B>
 where
     B: PartialOrd,
 {
+    #[inline]
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        let b: &B = self.borrow();
-        let other_b: &B = other.borrow();
-        b.partial_cmp(other_b)
+        self.internal_borrow().partial_cmp(other.internal_borrow())
     }
 }
 
@@ -219,14 +254,14 @@ impl<'b, B: 'b + ToOwned + ?Sized> Ord for Bos<'b, B>
 where
     B: Ord,
 {
+    #[inline]
     fn cmp(&self, other: &Self) -> Ordering {
-        let b: &B = self.borrow();
-        let other_b: &B = other.borrow();
-        b.cmp(other_b)
+        self.internal_borrow().cmp(other.internal_borrow())
     }
 }
 
 impl<'b, B: 'b + ToOwned + ?Sized> From<Cow<'b, B>> for Bos<'b, B> {
+    #[inline]
     fn from(c: Cow<'b, B>) -> Self {
         match c {
             Cow::Borrowed(x) => Self::Borrowed(x),
@@ -236,19 +271,22 @@ impl<'b, B: 'b + ToOwned + ?Sized> From<Cow<'b, B>> for Bos<'b, B> {
 }
 
 impl<'b, B: 'b + ToOwned + ?Sized> From<&'b B> for Bos<'b, B> {
-    fn from(b: &'b B) -> Self {
-        Self::Borrowed(b)
+    #[inline]
+    fn from(x: &'b B) -> Self {
+        Self::Borrowed(x)
     }
 }
 
-impl<'b, B: 'b + ToOwned + ?Sized> From<Arc<B>> for Bos<'b, B> {
-    fn from(a: Arc<B>) -> Self {
-        Self::Arc(a)
+impl<'b, B: 'b + ToOwned + ?Sized> From<Arc<B::Owned>> for Bos<'b, B> {
+    #[inline]
+    fn from(x: Arc<B::Owned>) -> Self {
+        Self::ArcOwned(x)
     }
 }
 
-impl<'b, B: 'b + ToOwned + ?Sized> From<Rc<B>> for Bos<'b, B> {
-    fn from(r: Rc<B>) -> Self {
-        Self::Rc(r)
+impl<'b, B: 'b + ToOwned + ?Sized> From<Rc<B::Owned>> for Bos<'b, B> {
+    #[inline]
+    fn from(x: Rc<B::Owned>) -> Self {
+        Self::RcOwned(x)
     }
 }
