@@ -3,6 +3,12 @@ This crate provides flexible **B**orrowed, **O**wned or **S**hared (B.O.S.) smar
 They are like std's [`Cow`][std::borrow::Cow]
 but with additonal [`Arc`][std::sync::Arc] and [`Rc`][std::rc::Rc] variants.
 
+The [`Abos`] and [`Bos`] smart pointers allow you to:
+
+- share data efficiently
+- decide ownership at runtime
+- avoid cloning the data until it becomes necessary
+
 # [`Abos`] or [`Bos`]
 
 [`Abos`] implements [`Send`] and [`Sync`] but does *not* have a [`Rc`][std::rc::Rc] variant.
@@ -21,7 +27,7 @@ fn maybe_replace_with_reply(string: &mut Abos<str>) {
     }
 }
 
-let mut string = Abos::Borrowed("Hello bos!");
+let mut string = Abos::ArcOwned("Hello bos!".to_owned().into());
 maybe_replace_with_reply(&mut string);
 assert_eq!(&string, "Hello to you, too.")
 ```
@@ -67,18 +73,12 @@ use std::{
     path::Path,
 };
 
-/// Atomic [`str`] smart pointer, that is **B**orrowed, **O**wned or **S**hared.
+/// **A**tomic [`str`] smart pointer, that is **B**orrowed, **O**wned or **S**hared.
 pub type AStr<'a> = Abos<'a, str>;
 
 /// [`str`] smart pointer, that is **B**orrowed, **O**wned or **S**hared.
 pub type Str<'a> = Bos<'a, str>;
 
-#[cfg(feature = "unstable")]
-pub type StaticAStr = Abos<'static, str>;
-
-#[cfg(feature = "unstable")]
-pub type StaticStr = Bos<'static, str>;
-
 #[cfg(feature = "unstable")]
 pub type AbosOsStr<'a> = Abos<'a, OsStr>;
 
